# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'FailedAccessAttempt.expired'
        db.add_column('cerberos_failedaccessattempt', 'expired',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'FailedAccessAttempt.expired'
        db.delete_column('cerberos_failedaccessattempt', 'expired')


    models = {
        'cerberos.failedaccessattempt': {
            'Meta': {'object_name': 'FailedAccessAttempt'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'expired': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'failed_logins': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'get_data': ('django.db.models.fields.TextField', [], {}),
            'http_accept': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_address': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'db_index': 'True'}),
            'locked': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'path_info': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'post_data': ('django.db.models.fields.TextField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'user_agent': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['cerberos']


# -*- coding: utf-8 -*-
from django.contrib.sites.models import Site
from django.db import models
#from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _ 
#from django.core.urlresolvers import reverse
#from django.contrib.auth.models import User
from cerberos.settings import MEMORY_FOR_FAILED_LOGINS
import datetime

try:
    from django.utils import timezone
except ImportError:
    from datetime import datetime as timezone

class FailedAccessAttempt(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    site = models.ForeignKey(Site, verbose_name=_(u'Site'))
    ip_address = models.IPAddressField(verbose_name=_(u'IP Address'), null=True, db_index=True)
    user_agent = models.CharField(max_length=255, verbose_name=_(u'User Agent'), blank=False,
            help_text=_(u'User agent used in the login attempt'))
    username = models.CharField(max_length=255, verbose_name=_(u'Username'), blank=False, db_index=True,
            help_text=_(u'Username used to login'))
    failed_logins = models.PositiveIntegerField(verbose_name=_(u'Failed logins'), default=0,
            help_text=_(u'Failed logins for this IP'))
    locked = models.BooleanField(verbose_name=_(u'Locked'), default=False, db_index=True,
            help_text=_(u'Indicates if the IP has been locked out.'))
    expired = models.BooleanField(verbose_name=_(u'Expired'), default=False, 
            help_text=_(u'Indicates if the timeout has expired.'))

    get_data = models.TextField('GET Data')
    post_data = models.TextField('POST Data')
    http_accept = models.TextField('HTTP Accept')
    path_info = models.CharField('Path', max_length=255)

    class Meta:
        verbose_name = _(u'Failed access attempt')
        verbose_name_plural = _(u'Failed access attempts')
        
    def __unicode__(self):
        return u'%s: %s' % (self.username, self.ip_address )

    def get_time_to_forget(self):
        """
        Returns the time until this access attempt is forgotten.
        """
        if MEMORY_FOR_FAILED_LOGINS > 0:
            now = timezone.now()
            delta = now - self.modified
            time_remaining = MEMORY_FOR_FAILED_LOGINS - delta.seconds
            return time_remaining
        else:
            return None

    def get_time_to_forget_text(self):
        """
        Returns the text for the admin based on the time to forget
        """
        time_remaining = self.get_time_to_forget()

        if not self.locked:
            return _(u'Not locked yet')
        elif time_remaining == None:
            return _(u'Infinite')
        elif time_remaining <= 0:
            return _(u'Forgotten')
        else:
            return _(u'%(time_remaining)s seconds' % {'time_remaining': time_remaining})
    get_time_to_forget.short_description = _(u'Time to forget')



# -*- coding: utf-8 -*-
from django.contrib.sites.models import Site
from cerberos.models import FailedAccessAttempt
from cerberos.settings import MAX_FAILED_LOGINS, MEMORY_FOR_FAILED_LOGINS
from django.shortcuts import render_to_response
from django.template import RequestContext
import datetime

def watch_logins(func):

    def new_func(request, *args, **kwargs):
        response = func(request, *args, **kwargs)
        ip = request.META.get('REMOTE_ADDR', '')

        failed_access = get_failed_access(ip)

        failed_access = check_failed_login(request, response, failed_access)

        if failed_access.locked:
            response = get_locked_response(request, ip, failed_access)

        return response
    return new_func

def get_failed_access(ip):
    """
    Returns the FailedAccessAttempt object for a given IP.
    """
    try:
        failed_access = FailedAccessAttempt.objects.get(ip_address=ip, expired=False)
    except FailedAccessAttempt.DoesNotExist:
        failed_access = None

    if failed_access:
        time_remaining = failed_access.get_time_to_forget()
        if time_remaining != None and time_remaining <= 0:
            failed_access.expired = True
            failed_access.save()
            return None

    return failed_access


def check_failed_login(request, response, failed_access):
    """
    If is a failed login, save the data in the database.

    It returns the FailedAccessAttempt instance.
    """
    site = Site.objects.get_current()
    ip = request.META.get('REMOTE_ADDR', '')
    user_agent = request.META.get('HTTP_USER_AGENT', 'unknown')
    username = request.POST.get('username')
    http_accept = request.META.get('HTTP_ACCEPT', 'unknown'),
    path_info = request.META.get('PATH_INFO', 'unknown')

    
    if not failed_access:
        failed_access = FailedAccessAttempt(ip_address=ip)

    if request.method == 'POST' and response.status_code != 302:
        # Failed login
        failed_access.site = site
        failed_access.user_agent = user_agent
        failed_access.username = username
        failed_access.failed_logins += 1
        failed_access.get_data = request.GET
        failed_access.post_data = request.POST
        failed_access.http_accept = http_accept
        failed_access.path_info = path_info

        if failed_access.failed_logins >= MAX_FAILED_LOGINS:
            # Lock the user
            failed_access.locked = True
        failed_access.save()
    elif request.method == 'POST' and response.status_code == 302 and failed_access.id and not failed_access.locked:
        # The user logged in successfully. Forgets about the access attempts
        failed_access.expired = True
        failed_access.save()

    return failed_access

def get_locked_response(request, ip, failed_access):
    return render_to_response('cerberos/user-locked.html',
                              {
                                  'ip':ip,
                                  'failed_access': failed_access,
                                  },
                              context_instance=RequestContext(request))



# -*- coding: utf-8 -*-
from django.contrib import admin
from cerberos.models import FailedAccessAttempt
from django.utils.translation import ugettext as _ 

class FailedAccessAttemptAdmin(admin.ModelAdmin):
    date_hierarchy = 'created'
    list_display = [
            'ip_address',
            'username',
            'locked',
            'expired',
            'user_agent',
            'failed_logins',
            'get_time_to_forget_text',
            'site',
            ]
    list_filter = [
            'locked',
            'expired',
            'site',
            ]
    search_fields = [
            'ip_address',
            'username',
            'user_agent',
            ]
    fieldsets = (
        ('Main data', {
            'fields': ('site', 'ip_address', 'username', 'locked', 'expired', 'failed_logins', )
        }),
        ('Data recollected', {
            #'classes': ('collapse',),
            'fields': ('user_agent', 'get_data', 'post_data', 'http_accept', 'path_info',)
        }),
    )
    actions = ['lock', 'unlock']

    def lock(self, request, queryset):
        queryset.update(locked=True)
    lock.short_description = _(u'Lock the users')

    def unlock(self, request, queryset):
        queryset.update(locked=False)
    unlock.short_description = _(u'Unlock the users')

admin.site.register(FailedAccessAttempt, FailedAccessAttemptAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import subprocess
from setuptools import setup, find_packages


version = '0.2.0'


if sys.argv[-1] == 'publish':
    subprocess.call(['python', 'setup.py', 'sdist', 'upload'])
    print "You probably want to also tag the version now:"
    print "  git tag -a %s -m 'Tag version %s'" % (version, version)
    print "  git push --tags"
    sys.exit()

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]

setup(
    name='cerberos',
    version=version,
    description='Cerberos is a django app that watches failed logins and block the user after N attempts.',
    author='Adrián Ribao',
    url='https://github.com/AdrianRibao/cerberos',
    packages = find_packages(exclude=['tests', 'tests.*']),
    include_package_data = True,
    license='BSD',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[
    ],
)



from django.conf import settings

MAX_FAILED_LOGINS = getattr(settings, 'MAX_FAILED_LOGINS', 3)

# Number of seconds after the failed access attempts are forgotten.
MEMORY_FOR_FAILED_LOGINS = getattr(settings, 'MEMORY_FOR_FAILED_LOGINS', 0)
